@@ -1,13 +1,14 @@
 from dateutil.parser import parse as parse_date
 from django.db import models
+from django.template.loader import render_to_string
 from django.urls import reverse
 from django.utils import html, text
 from feedparser import parse as parse_feed
 from html2text import html2text
 from markdownx.models import MarkdownxField
+from pico.conf import settings
 from urllib.parse import urlparse
-from django.template.loader import render_to_string
-from .query import PodcastQuerySet, EpisodeQuerySet
+from .query import PodcastQuerySet, EpisodeQuerySet, PostQuerySet
 from .utils import download, compare_image
 import os
 import re
@@ -31,8 +32,9 @@ class Podcast(models.Model):
         )
 
     name = models.CharField(max_length=100)
+    short_name = models.CharField(max_length=100, null=True, blank=True)
     slug = models.SlugField(max_length=30, unique=True)
-    domain = models.CharField(max_length=100, unique=True)
+    domain = models.CharField(max_length=100, null=True, blank=True)
     rss_feed_url = models.URLField('RSS feed URL', max_length=255, unique=True)
 
     artwork = models.ImageField(
@@ -151,6 +153,24 @@ class Podcast(models.Model):
 
         return self.__apple_podcasts_id
 
+    def reverse(self, urlname, args=(), kwargs={}):
+        if settings.DOMAINS_OR_SLUGS == 'slugs':
+            if any(kwargs):
+                kwargs['podcast'] = self.slug
+            else:
+                args = (self.slug,) + args
+
+        return reverse(urlname, args=args, kwargs=kwargs)
+
+    def build_absolute_uri(self, path=''):
+        while path.startswith('/'):
+            path = path[1:]
+
+        if settings.DOMAINS_OR_SLUGS == 'slugs':
+            return '/%s/%s' % (self.slug, path)
+
+        return '//%s/%s' % (self.domain, path)
+
     class Meta:
         ordering = ('name',)
 
@@ -232,7 +252,7 @@ class Season(models.Model):
         return self.name or ('Season %d' % self.number)
 
     def get_absolute_url(self):
-        return reverse('season', args=(self.number,))
+        return self.podcast.reverse('season', args=(self.number,))
 
     class Meta:
         unique_together = ('number', 'podcast')
@@ -275,12 +295,6 @@ class Category(models.Model):
     def __str__(self):
         return self.name
 
-    def get_absolute_url(self):
-        return '%s?category=%s' % (
-            reverse('episode_list'),
-            self.slug
-        )
-
     class Meta:
         ordering = ('name',)
         verbose_name_plural = 'categories'
@@ -361,7 +375,7 @@ class Episode(models.Model):
     def get_absolute_url(self):
         if self.season_id:
             if self.trailer:
-                return reverse(
+                return self.podcast.reverse(
                     'season_episode_trailer_detail',
                     args=(
                         self.season.number,
@@ -369,7 +383,7 @@ class Episode(models.Model):
                 )
 
             if self.bonus:
-                return reverse(
+                return self.podcast.reverse(
                     'season_episode_bonus_detail',
                     args=(
                         self.season.number,
@@ -377,7 +391,7 @@ class Episode(models.Model):
                     )
                 )
 
-            return reverse(
+            return self.podcast.reverse(
                 'season_episode_detail',
                 args=(
                     self.season.number,
@@ -386,12 +400,14 @@ class Episode(models.Model):
             )
 
         if self.trailer:
-            return reverse('episode_trailer_detail')
+            return self.podcast.reverse('episode_trailer_detail')
 
         if self.bonus:
-            return reverse('episode_bonus_detail', args=(self.number,))
+            return self.podcast.reverse(
+                'episode_bonus_detail', args=(self.number,)
+            )
 
-        return reverse('episode_detail', args=(self.number,))
+        return self.podcast.reverse('episode_detail', args=(self.number,))
 
     @property
     def player_html(self):
@@ -413,9 +429,17 @@ class Episode(models.Model):
 
 
 class Post(models.Model):
+    objects = PostQuerySet.as_manager()
+
     def upload_image(self, filename):
-        return 'podcasts/%s/blog/%s%s' % (
-            self.podcast.slug,
+        if self.podcast:
+            return 'podcasts/%s/blog/%s%s' % (
+                self.podcast.slug,
+                self.slug,
+                os.path.splitext(filename)[-1]
+            )
+
+        return 'blog/%s%s' % (
             self.slug,
             os.path.splitext(filename)[-1]
         )
@@ -423,7 +447,9 @@ class Post(models.Model):
     podcast = models.ForeignKey(
         Podcast,
         on_delete=models.CASCADE,
-        related_name='blog_posts'
+        related_name='blog_posts',
+        null=True,
+        blank=True
     )
 
     author = models.ForeignKey(
@@ -455,7 +481,10 @@ class Post(models.Model):
         return self.title
 
     def get_absolute_url(self):
-        return reverse('blogpost_detail', args=(self.slug,))
+        if self.podcast_id and settings.DOMAINS_OR_SLUGS == 'slugs':
+            return '/%s/blog/%s/' % (self.podcast.slug, self.slug)
+
+        return '/blog/%s/' % self.slug
 
     class Meta:
         unique_together = ('slug', 'podcast')
@@ -464,8 +493,14 @@ class Post(models.Model):
 
 class Page(models.Model):
     def upload_image(self, filename):
-        return 'podcasts/%s/pages/%s%s' % (
-            self.podcast.slug,
+        if self.podcast:
+            return 'podcasts/%s/pages/%s%s' % (
+                self.podcast.slug,
+                self.slug,
+                os.path.splitext(filename)[-1]
+            )
+
+        return 'pages/%s%s' % (
             self.slug,
             os.path.splitext(filename)[-1]
         )
@@ -473,7 +508,9 @@ class Page(models.Model):
     podcast = models.ForeignKey(
         Podcast,
         on_delete=models.CASCADE,
-        related_name='pages'
+        related_name='pages',
+        null=True,
+        blank=True
     )
 
     title = models.CharField(max_length=255)
@@ -501,7 +538,10 @@ class Page(models.Model):
         return self.title
 
     def get_absolute_url(self):
-        return reverse('page_detail', args=(self.slug,))
+        if self.podcast_id and settings.DOMAINS_OR_SLUGS == 'slugs':
+            return '/%s/%s/' % (self.podcast.slug, self.slug)
+
+        return '/%s/' % self.slug
 
     class Meta:
         unique_together = ('slug', 'podcast')
