@@ -1,19 +1,51 @@
 from django.http.response import HttpResponsePermanentRedirect
 from django.shortcuts import get_object_or_404
-from django.urls import reverse
 from django.utils import timezone
 from django.views.generic.base import View
 from django.views.generic.detail import DetailView
 from django.views.generic.list import ListView
+from pico.conf import settings
 from pico.seo.mixins import (
     SEOMixin, OpenGraphMixin, OpenGraphArticleMixin
 )
 
-from .models import Episode, Season, Post, Page, Category
+from .models import Podcast, Episode, Season, Post, Page, Category
 
 
 class PodcastMixin(object):
     def get_menu_items(self):
+        if settings.DOMAINS_OR_SLUGS == 'slugs':
+            yield {
+                'url': self.request.build_absolute_uri('/'),
+                'text': 'Home'
+            }
+
+            for podcast in Podcast.objects.all():
+                yield {
+                    'url': podcast.build_absolute_uri(),
+                    'text': podcast.short_name or podcast.name
+                }
+
+            for page in Page.objects.filter(
+                podcast=None,
+                menu_visible=True
+            ):
+                yield {
+                    'url': page.get_absolute_url(),
+                    'text': page.menu_title or page.title
+                }
+
+            if Post.objects.filter(
+                published__lte=timezone.now(),
+                podcast=None
+            ).exists():
+                yield {
+                    'url': '/blog/',
+                    'text': 'Blog'
+                }
+
+            return
+
         yield {
             'url': '/',
             'text': 'Home'
@@ -21,19 +53,27 @@ class PodcastMixin(object):
 
         for season in self.request.podcast.seasons.order_by('-number'):
             yield {
-                'url': reverse('season', args=(season.number,)),
+                'url': self.request.podcast.reverse(
+                    'season',
+                    args=(season.number,)
+                ),
                 'text': str(season)
             }
 
-        if self.request.podcast.blog_posts.exists():
+        if self.request.podcast.blog_posts.filter(
+            published__lte=timezone.now()
+        ).exists():
             yield {
-                'url': reverse('blogpost_list'),
+                'url': self.request.podcast.reverse('blogpost_list'),
                 'text': 'Blog'
             }
 
         for page in self.request.podcast.pages.filter(menu_visible=True):
             yield {
-                'url': reverse('page_detail', args=(page.slug,)),
+                'url': self.request.podcast.reverse(
+                    'page_detail',
+                    args=(page.slug,)
+                ),
                 'text': page.menu_title or page.title,
                 'highlight': page.cta
             }
@@ -50,21 +90,34 @@ class PodcastMixin(object):
         return items
 
     def get_context_data(self, **kwargs):
+        base_url = self.request.build_absolute_uri('/')
+        if settings.DOMAINS_OR_SLUGS == 'slugs' and self.request.podcast:
+            base_url = self.request.podcast.build_absolute_uri('/')
+
         return {
             'podcast': self.request.podcast,
-            'base_url': self.request.build_absolute_uri('/'),
+            'base_url': base_url,
             'menu_items': self.build_menu(),
             **super().get_context_data(**kwargs)
         }
 
     def get_og_title(self):
-        return self.request.podcast.name
+        if self.request.podcast:
+            return self.request.podcast.name
+
+        return super().get_og_title()
 
     def get_og_description(self):
-        return self.request.podcast.subtitle
+        if self.request.podcast:
+            return self.request.podcast.subtitle
+
+        return super().get_og_description()
 
     def get_og_site_name(self):
-        return self.request.podcast.name
+        if self.request.podcast:
+            return self.request.podcast.name
+
+        return super().get_og_site_name()
 
     def get_twitter_card(self):
         return self.twitter_card
@@ -76,7 +129,7 @@ class PodcastMixin(object):
         return self.get_og_description()
 
     def get_twitter_creator(self):
-        if self.request.podcast.twitter_username:
+        if self.request.podcast and self.request.podcast.twitter_username:
             return '@%s' % self.request.podcast.twitter_username
 
         return super().get_twitter_creator()
@@ -88,10 +141,13 @@ class EpisodeListView(PodcastMixin, SEOMixin, OpenGraphMixin, ListView):
     paginate_by = 10
 
     def get_seo_title(self):
-        return '%s – %s' % (
-            self.request.podcast.name,
-            self.request.podcast.subtitle
-        )
+        if self.request.podcast.subtitle:
+            return '%s – %s' % (
+                self.request.podcast.name,
+                self.request.podcast.subtitle
+            )
+
+        return self.request.podcast.name
 
     def get_queryset(self):
         queryset = super().get_queryset().filter(
@@ -120,12 +176,12 @@ class EpisodeListView(PodcastMixin, SEOMixin, OpenGraphMixin, ListView):
             for category in Category.objects.filter(
                 slug=self.request.GET['category']
             ):
-                return self.request.build_absolute_uri(
-                    category.get_absolute_url()
+                return self.request.podcast.build_absolute_uri(
+                    '?category=%s' % category.slug
                 )
 
         return self.request.build_absolute_uri(
-            reverse('episode_list')
+            self.request.podcast.reverse('episode_list')
         )
 
 
@@ -150,7 +206,7 @@ class SeasonView(EpisodeListView):
 
     def get_canonical_url(self):
         return self.request.build_absolute_uri(
-            reverse('season', kwargs=self.kwargs)
+            self.request.podcast.reverse('season', kwargs=self.kwargs)
         )
 
 
@@ -217,7 +273,10 @@ class PostListView(PodcastMixin, SEOMixin, OpenGraphMixin, ListView):
     paginate_by = 10
 
     def get_seo_title(self):
-        return 'Blog – %s' % self.request.podcast.subtitle
+        if self.request.podcast and self.request.podcast.subtitle:
+            return 'Blog – %s' % self.request.podcast.subtitle
+
+        return 'Blog'
 
     def get_queryset(self):
         queryset = super().get_queryset().filter(
@@ -243,9 +302,12 @@ class PostListView(PodcastMixin, SEOMixin, OpenGraphMixin, ListView):
         return context
 
     def get_canonical_url(self):
-        return self.request.build_absolute_uri(
-            reverse('blogpost_list')
-        )
+        if self.request.podcast:
+            return self.request.build_absolute_uri(
+                self.request.podcast.reverse('blogpost_list')
+            )
+
+        return self.request.build_absolute_uri('/blog/')
 
 
 class PostDetailView(
@@ -254,10 +316,13 @@ class PostDetailView(
     model = Post
 
     def get_seo_title(self):
-        return '%s | %s' % (
-            self.object.title,
-            self.request.podcast.name
-        )
+        if self.request.podcast:
+            return '%s | %s' % (
+                self.object.title,
+                self.request.podcast.name
+            )
+
+        return self.object.title
 
     def get_seo_description(self):
         return self.object.summary
@@ -272,7 +337,8 @@ class PostDetailView(
         if self.object.image:
             return self.object.image
 
-        return self.object.podcast.artwork
+        if self.object.podcast:
+            return self.object.podcast.artwork
 
     def get_queryset(self):
         return super().get_queryset().filter(
@@ -286,10 +352,13 @@ class PageDetailView(
     model = Page
 
     def get_seo_title(self):
-        return '%s | %s' % (
-            self.object.title,
-            self.request.podcast.name
-        )
+        if self.object.podcast_id:
+            return '%s | %s' % (
+                self.object.title,
+                self.object.podcast.name
+            )
+
+        return self.object.title
 
     def get_og_title(self):
         return self.object.title
@@ -298,7 +367,8 @@ class PageDetailView(
         if self.object.image:
             return self.object.image
 
-        return self.object.podcast.artwork
+        if self.object.podcast_id:
+            return self.object.podcast.artwork
 
     def get_queryset(self):
         return super().get_queryset().filter(
