@@ -1,8 +1,9 @@
-from django.conf import settings
+from django.conf import settings as django
 from django.contrib import admin
 from django.urls import path, re_path, include
 from django.views.decorators.csrf import csrf_exempt
-from .views import PodcastListView, PodcastPingView
+from pico.conf import settings as pico
+from .views import PodcastListView, PodcastPingView, ContentListView
 
 
 urlpatterns = (
@@ -10,11 +11,11 @@ urlpatterns = (
     path('admin/', admin.site.urls),
     path('markdownx/', include('markdownx.urls')),
     path('~/ping/', csrf_exempt(PodcastPingView.as_view()), name='api_ping'),
+    path('~/content/', ContentListView.as_view(), name='content_list'),
     path('', PodcastListView.as_view(), name='podcast_list')
 )
 
-
-if settings.DEBUG:
+if django.DEBUG:
     from django.views.static import serve as static_serve
 
     urlpatterns += (
@@ -22,7 +23,13 @@ if settings.DEBUG:
             r'^media/(?P<path>.*)$',
             static_serve,
             {
-                'document_root': settings.MEDIA_ROOT
+                'document_root': django.MEDIA_ROOT
             }
         ),
     )
+
+
+if pico.DOMAINS_OR_SLUGS == 'slugs':
+    urlpatterns += (
+        path('<podcast>/', include('pico.podcasts.urls')),
+    )
